function love.load()

	setShorcuts()
	setMouse()
	setBackground()
	setObjects()
	setBallPS()
	setColors()
end

function love.update(dt)

	bouncePaddle(
	
		ball.x,
		ball.y,
		ball.angle,
		paddle.x,
		paddle.y,
		paddle.width,
		paddle.height
	)
	
	bounceWall(
	
		ball.x,
		ball.y,
		ball.angle,
		wall.left,
		wall.right,
		wall.up
	)

	bounceBricks()

	moveBall(
	
		ball.x,
		ball.y,
		ball.angle,
		ball.vel,
		dt
	)
	
	movePaddle(
	
		mo.getX(),
		paddle.width,
		wall.left,
		wall.right
	)
	
	lostLife()
end

function love.draw()

	drawBackground()

	drawBricks()

	drawBall()

	drawWall(
	
		wall.left,
		wall.right,
		wall.up)
		
	drawPaddle(
	
		paddle.x,
		paddle.y,
		paddle.width,
		paddle.height
	)

	info(
	
		ball.x,
		ball.y,
		ball.angle,
		ball.vel,
		paddle.x
	)
end

function love.keypressed(key)

	if key == "escape"
	
	or key == "return" then
	
		love.event.quit()
	end
end

function setShorcuts()

	mo = love.mouse
	gr = love.graphics
	
	getW = gr.getWidth()
	getH = gr.getHeight()
end

function setMouse()

	mo.setVisible(false)
	mo.setGrab(true)
end

function setBackground()

	gr.setBackgroundColor(0x20, 0x30, 0x40)
	background = gr.newImage("background.jpg")
end

function setObjects()

	paddle = {

		x = getW/2;
		y = getH - 0x40;
		width = 0x80;
		height = 0x08;
	}
	
	ball = {

		x = getW/2;
		y = getH - 0x40;
		vel = 0x200;
		angle = - math.pi/3;
	}
	
	wall = {

		left = 8;
		right = 8;
		up = 8;
	}
	
	bricks = {}
	
	bricks.w = 64
	bricks.h = 32
	
	for x = 64, getW-128, bricks.w do
	
		for y = 64, getH-256, bricks.h do
		
			brick = {
			
				x = x,
				y = y,
				life = 1,
			}
			
			table.insert(bricks, brick)
		end
	end
end

function setColors()

	color = {

		paddle = {
		
			fill = {
			
				0x40, 0x60, 0x80, 0xC0
			};
			
			line = {
			
				0x40, 0x80, 0xff, 0xE0
			};
		};
		
		info = {
		
			0xff, 0xff, 0xff
		};
		
		wall = {
		
			left = {
			
				0x40, 0x60, 0x80, 0x80
			};
			
			right = {
			
				0x40, 0x60, 0x80, 0x80
			};
			
			up = {
			
				0x40, 0x60, 0x80, 0x80
			};
		};
	}
	
	ball_ps:setColors(
		0x80, 0x60, 0x40, 0xff,
		0x40, 0x00 , 0x00 , 0x00
	)
end

function setBallPS()

	part = gr.newImage("part.png")
	
	ball_ps =
		gr.newParticleSystem(
			part, 0x20
		)
	
	ball_ps:setEmissionRate(0x80)
	ball_ps:setParticleLife(.2)
	
	ball_ps:setSizes(1, .5)
	ball_ps:setSpeed(0x0, 0x80)
	ball_ps:setSpread(15)
end

function moveBall(x, y, a, v, dt)

	ball.x = x + math.cos(a) * v * dt
	ball.y = y + math.sin(a) * v * dt
	
	ball_ps:setPosition(
	
		ball.x,
		ball.y
	)
	
	ball_ps:update(dt)
end

function bounceWall(x, y, a, l, r, u)

	if x < l then

		ball.x = l
		ball.angle = math.pi - a
		
	elseif x > getW - r then
	
		ball.x = getW - r
		ball.angle = math.pi - a
	end
	
	if y < u then

		ball.y = u
		ball.angle = - a
	end
end

function movePaddle(mx, w, l, r)
	
	local p = mx - w/2
	
	if p < l then
	
		p = l
	
	elseif p + w > getW - r then
	
		p = getW - r - w
	end
	
	paddle.x = p
end

function bouncePaddle(bx, by, ba, px, py, pw, ph)

	if by > py then
	
		if by < py + ph * 2 then
		
			if bx > px
			
			and bx < px + pw then
							
				ball.y = py
				ball.angle = - ba - ((px+pw/2-bx)/pw/2) * math.pi
			end
		end
	end
end

function lostLife()

	if ball.y > getH then

		newLife()
	end
end

function newLife()

	ball.x = paddle.x + paddle.width/2
	ball.y = paddle.y
	ball.angle = math.random(-2*math.pi/3, -math.pi/3) -- !
end

function drawBall()

	gr.setColorMode("modulate")
	gr.setBlendMode("additive")
	
	gr.draw(ball_ps, 0, 0)
end

function drawWall(l, r, u)

	gr.setColorMode("modulate")
	gr.setBlendMode("alpha")

	gr.setColor(color.wall.left)

	gr.rectangle(
		"fill",
		0, 0,
		l, getH
	)

	gr.setColor(color.wall.right)

	gr.rectangle(
		"fill",
		getW - r, 0,
		r, getH
	)
	
	gr.setColor(color.wall.up)

	gr.rectangle(
		"fill",
		l, 0,
		getW - ( l + r ), u
	)
end

function drawPaddle(x, y, w, h)

	gr.setColorMode("modulate")
	gr.setBlendMode("alpha")

	gr.setColor(color.paddle.fill)

	gr.rectangle(
		"fill",
		x, y,
		w, h
	)
	
	gr.setColor(0, 0, 0, 255)

	gr.setLineWidth(4)
	
	gr.rectangle(
		"line",
		x, y,
		w, h
	)

	gr.setColor(color.paddle.line)
	
	gr.setLineWidth(2)
	
	gr.rectangle(
		"line",
		x, y,
		w, h
	)
end

function drawBackground()

	gr.setColorMode("modulate")
	gr.setBlendMode("alpha")

	gr.draw(background, 0, 0)

end

function drawBricks()

	for k, v in ipairs(bricks) do
	
		if bricks[k].life > 0 then
			gr.setColor(255, 96, 64, 128)
			gr.rectangle("fill", bricks[k].x, bricks[k].y, bricks.w, bricks.h)
			gr.setLineWidth(1)
			gr.setColor(32, 16, 16, 192)
			gr.rectangle("line", bricks[k].x, bricks[k].y, bricks.w, bricks.h)
		end
	end
end

function bounceBricks()

	for k, v in ipairs(bricks) do
		
		if ball.x > bricks[k].x
		and ball.x < bricks[k].x + 64 then
		
			if ball.y > bricks[k].y
			and ball.y < bricks[k].y + 32 then
		
				bricks[k].life = bricks[k].life - 1
			end
		end
	end
end

function info(bx, by, ba, bv, px)

	gr.setColorMode("modulate")
	gr.setBlendMode("alpha")
	
	gr.setColor(color.info)

	gr.print(
	
		"FPS : "..
		love.timer.getFPS()..
		"\nparticules "..
		ball_ps:count()..
		"\nball.x "..
		math.floor(bx)..
		"\nball.y "..
		math.floor(by)..
		"\nball.angle "..
		ba..
		"\nball.v "..
		bv..
		"\npaddle.x "..
		math.floor(px),
		32, 32
	)	
end
